from datetime import datetime

import click
from flask.cli import with_appcontext
from werkzeug.security import generate_password_hash

from ..helper import db_add, db_commit, db_delete, db_update, get_object_model
from ..validators import validate_email, validate_password_length, validate_username


@click.group("user")
def group():
    """
    User manager.
    """


@group.command()
@with_appcontext
@click.argument("email")
@click.option("-f", "--first-name", required=True, help="First name.", prompt=True)
@click.option("-l", "--last-name", required=True, help="Last name.", prompt=True)
@click.option(
    "-p",
    "--password",
    required=True,
    help="Password.",
    prompt=True,
    hide_input=True,
    confirmation_prompt=True,
)
@click.option("-r", "--roles", help="User roles (separated by ,).")
def new(email, first_name, last_name, password, roles):
    """
    Create user.
    """

    validate_email(email)
    validate_username(first_name)
    validate_username(last_name)
    validate_password_length(password)
    if not roles:
        roles = input("User roles (separated by ,): ").strip()

    role_model = get_object_model("role")
    user_roles = []
    for role in roles.split(","):
        role = role_model.query.filter_by(name=role.strip()).first()
        if role:
            user_roles.append(role)

    model = get_object_model("user")
    user = model.query.filter_by(email=email).first()
    if not user:
        username = first_name + " " + last_name
        passw = generate_password_hash(password)
        user = model(
            first_name=first_name,
            last_name=last_name,
            name=username,
            email=email,
            password=passw,
            register_at=datetime.utcnow(),
            confirmed=True,
            confirmed_at=datetime.utcnow(),
            roles=user_roles,
        )
        db_add(user)
        print("Successfully added user %r" % email)
    else:
        print("Email %r already exists" % email)


@group.command()
@with_appcontext
@click.argument("email")
def remove(email):
    """
    Remove user.
    """

    model = get_object_model("user")
    user = model.query.filter_by(email=email).first()
    if user:
        db_delete(user)
        print("Successfully deleted user %r" % email)
    else:
        print("User %r doesn't exist" % email)


@group.command()
@with_appcontext
@click.argument("email")
def update(email):
    """
    Update user.
    """

    model = get_object_model("user")
    role_model = get_object_model("role")
    user = model.query.filter_by(email=email).first()
    if user:
        cols = {}
        first_name = input("New first name: ").strip()
        if not validate_username(first_name, silently=True):
            first_name = user.first_name
        cols["first_name"] = first_name

        last_name = input("New last name: ").strip()
        if not validate_username(last_name, silently=True):
            last_name = user.last_name
        cols["last_name"] = last_name

        new_email = input("New email: ").strip()
        if not validate_email(new_email, silently=True):
            new_email = user.email
        cols["email"] = new_email

        password = input("New password: ").strip()
        if password:
            validate_password_length(password)
            password = generate_password_hash(password)
        else:
            password = user.password
        cols["password"] = password
        cols["name"] = cols["first_name"] + " " + cols["last_name"]
        roles = input("Enter a new user role (separated by ,): ")
        user_roles = []
        for role in roles.split(","):
            role = role_model.query.filter_by(name=role.strip()).first()
            if role:
                user_roles.append(role)

        if user_roles:
            cols["roles"] = user_roles

        db_update(user, **cols)
        print("User %r updated successfully" % email)

    else:
        print("User %r doesn't exist" % email)


@group.command()
@with_appcontext
def list():
    """
    Show users.
    """

    model = get_object_model("user")
    users = model.query.all()
    print("Total users: %d" % len(users))
    for user in users:
        roles = []
        for role in user.roles:
            roles.append(role.name)

        print("* %s (%s) [%s]" % (user.name, user.email, ", ".join(roles)))


@group.command()
@with_appcontext
def drop():
    """
    Drop users.
    """

    print("Dropping all users... ", end="")
    model = get_object_model("user")
    model.query.delete()
    db_commit()
    print("(done)")


command = group



from zemfrog.globals import db
from zemfrog.mixins import UserMixin, RoleMixin, PermissionMixin, LogMixin


class User(UserMixin, db.Model):
    pass


class Role(RoleMixin, db.Model):
    pass


class Permission(PermissionMixin, db.Model):
    pass


class Log(LogMixin, db.Model):
    pass



from importlib import import_module

from flask import Flask

from zemfrog.globals import apispec as docs

from ..decorators import api_doc
from ..helper import get_import_name


def loader(app: Flask):
    """
    Function for creating api docs using ``flask-apispec``.
    """

    if not app.config.get("API_DOCS", False):
        return

    import_name = get_import_name(app)
    urls = import_module(import_name + "urls")
    docs_params = urls.docs
    routes = urls.routes
    for _, view, _ in routes:
        if docs_params:
            view = api_doc(**docs_params)(view)
        docs.register(view)

    apis = app.config.get("APIS", [])
    api_prefix = "apis."
    for name in apis:
        if not name.startswith(api_prefix):
            name = api_prefix + name

        try:
            res = import_module(import_name + name)
        except ImportError:
            res = import_module(name.lstrip(api_prefix))

        docs_params = res.docs
        routes = res.routes
        endpoint = res.endpoint
        for detail in routes:
            _, view, _ = detail
            e = endpoint + "_" + view.__name__
            if docs_params:
                view = api_doc(**docs_params)(view)
            docs.register(view, endpoint=e, blueprint="api")

    blueprints = app.config.get("BLUEPRINTS", [])
    for name in blueprints:
        urls = name + ".urls"
        try:
            urls = import_name + urls
            urls = import_module(urls)
        except (ImportError, AttributeError):
            urls = import_module(urls)

        docs_params = urls.docs
        routes = urls.routes

        for _, view, _ in routes:
            if docs_params:
                view = api_doc(**docs_params)(view)
            docs.register(view, blueprint=name)



from flask import Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    Function to load all error handlers
    """

    import_name = get_import_name(app)
    handlers = app.config.get("ERROR_HANDLERS", {})
    prefix = "handlers."
    for code, func in handlers.items():
        if not func.startswith(prefix):
            func = prefix + func
        func += ".handler"
        try:
            view = import_attr(import_name + func)
        except (ImportError, AttributeError):
            view = import_attr(func.lstrip(prefix))

        app.register_error_handler(code, view)



from flask import Blueprint, Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    The function to load all blueprints based on the ``BLUEPRINTS`` configuration in config.py
    """

    blueprints = app.config.get("BLUEPRINTS", [])
    import_name = get_import_name(app)
    for name in blueprints:
        bp = name + ".routes.blueprint"
        routes = name + ".urls.routes"
        try:
            bp: Blueprint = import_attr(import_name + bp)
            routes = import_attr(import_name + routes)
        except (ImportError, AttributeError):
            bp: Blueprint = import_attr(bp)
            routes = import_attr(routes)

        for url, view, methods in routes:
            bp.add_url_rule(url, view_func=view, methods=methods)

        app.register_blueprint(bp)



class ZemfrogException(Exception):
    pass


class ZemfrogTemplateNotFound(ZemfrogException):
    pass


class ZemfrogEnvironment(ZemfrogException):
    pass


class ZemfrogModelNotFound(ZemfrogException):
    pass


class ZemfrogRoleNotFound(ZemfrogException):
    pass


class ZemfrogRolePermissionNotFound(ZemfrogException):
    pass



from flask import Flask, jsonify
from flask_jwt_extended import JWTManager

from ..helper import get_object_model


def init_app(app: Flask):
    User = get_object_model("user")
    jwt = JWTManager()

    @jwt.user_loader_callback_loader
    def user_loader_callback(identity):
        user = User.query.filter_by(email=identity).first()
        if user and not user.confirmed:
            user = None

        return user

    @jwt.user_loader_error_loader
    def custom_user_loader_error(identity):
        ret = {"reason": "User {} not found".format(identity), "status_code": 404}
        return jsonify(ret), 404

    jwt.init_app(app)



import click

from . import __version__
from .generator import g_project


@click.group()
@click.version_option(__version__, "-v", "--version", message="Zemfrog v%(version)s")
def main():
    """
    Zemfrog CLI
    """


@main.command("create")
@click.argument("name")
def create(name):
    """
    Create a project.
    """

    g_project(name, "wsgi")



from datetime import datetime, timedelta
from flask_jwt_extended import create_access_token, decode_token, get_raw_jwt
from flask_apispec import use_kwargs, marshal_with
from jwt import DecodeError, ExpiredSignatureError
from marshmallow import fields
from werkzeug.security import generate_password_hash, check_password_hash
from zemfrog.decorators import http_code, authenticate
from zemfrog.helper import db_add, db_update, db_commit, get_mail_template, get_user_roles
from zemfrog.models import (
    DefaultResponseSchema,
    LoginSchema,
    LoginSuccessSchema,
    PasswordResetSchema,
    RegisterSchema,
    RequestPasswordResetSchema,
)

from zemfrog.globals import ma
from {{ "" if main_app else ".." }}models.user import User, Log, Role, Permission
from {{ "" if main_app else ".." }}tasks.mail import send_email


class PermissionSchema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = Permission


class RoleSchema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = Role

    permissions = fields.List(fields.Nested(PermissionSchema()))


class UserDetailSchema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = User
        exclude = ("password",)

    roles = fields.List(fields.Nested(RoleSchema()))


@authenticate()
@marshal_with(UserDetailSchema)
def user_detail():
    """
    User detail info.
    """

    email = get_raw_jwt().get("identity")
    user = User.query.filter_by(email=email).first()
    return user

@use_kwargs(LoginSchema(), location="form")
@marshal_with(DefaultResponseSchema, 404)
@marshal_with(LoginSuccessSchema, 200)
@http_code
def login(**kwds):
    """
    Login and get access token.
    """

    email = kwds.get("username")
    passw = kwds.get("password")
    user = User.query.filter_by(email=email).first()

    if user and user.confirmed and check_password_hash(user.password, passw):
        login_at = datetime.utcnow()
        log = Log(login_at=login_at)
        user.logs.append(log)
        db_commit()
        roles = get_user_roles(user)
        claims = {"roles": roles}
        access_token = create_access_token(email, user_claims=claims)
        return {"access_token": access_token}

    return {"message": "Incorrect email or password.", "code": 404}


@use_kwargs(RegisterSchema(), location="form")
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 403)
@http_code
def register(**kwds):
    """
    Register an account.
    """

    email = kwds.get("username")
    passw = kwds.get("password")
    first_name = kwds.get("first_name")
    last_name = kwds.get("last_name")
    username = first_name + " " + last_name
    if email:
        user = User.query.filter_by(email=email).first()
        if not user:
            if username and passw:
                passw = generate_password_hash(passw)
                user = User(
                    first_name=first_name,
                    last_name=last_name,
                    name=username,
                    email=email,
                    password=passw,
                    register_at=datetime.utcnow(),
                )
                db_add(user)
                token = create_access_token(
                    email,
                    expires_delta=False,
                    user_claims={"token_registration": True},
                )
                msg = get_mail_template("register.html", token=token)
                send_email.delay("Registration", html=msg, recipients=[email])
                message = "Successful registration."
                status_code = 200
            else:
                message = "Username and password are required."
                status_code = 403
        else:
            message = "Email already exists."
            status_code = 403
    else:
        message = "Email required."
        status_code = 403

    return {"message": message, "code": status_code}


@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 403)
@http_code
def confirm_account(token):
    """
    Confirm account.
    """

    try:
        data = decode_token(token)
        email = data["identity"]
        if not data["user_claims"].get("token_registration"):
            raise DecodeError

        user = User.query.filter_by(email=email).first()
        if user and not user.confirmed:
            message = "Confirmed."
            status_code = 200
            db_update(user, confirmed=True, confirmed_at=datetime.utcnow())

        else:
            raise DecodeError

    except DecodeError:
        message = "Invalid token."
        status_code = 403

    return {"message": message, "code": status_code}


@use_kwargs(RequestPasswordResetSchema(), location="form")
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 404)
@marshal_with(DefaultResponseSchema, 403)
@http_code
def request_password_reset(**kwds):
    """
    Request a password reset.
    """

    email = kwds.get("username")
    if email:
        user = User.query.filter_by(email=email).first()
        if not user:
            message = "User not found."
            status_code = 404
        else:
            message = "A password reset request has been sent."
            status_code = 200
            token = create_access_token(
                email,
                expires_delta=timedelta(hours=2),
                user_claims={"token_password_reset": True},
            )
            msg = get_mail_template(
                "request_password_reset.html", token=token
            )
            send_email.delay("Forgot password", html=msg, recipients=[email])
            log = Log(request_password_reset_at=datetime.utcnow())
            user.logs.append(log)
            db_commit()
    else:
        message = "Email required."
        status_code = 403

    return {"message": message, "code": status_code}


@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 401)
@marshal_with(DefaultResponseSchema, 403)
@http_code
def confirm_password_reset_token(token):
    """
    Validate password reset token.
    """

    try:
        data = decode_token(token)
        email = data["identity"]
        if not data["user_claims"].get("token_password_reset"):
            raise DecodeError

        user = User.query.filter_by(email=email).first()
        if user:
            message = "Valid token."
            status_code = 200
        else:
            raise DecodeError

    except DecodeError:
        message = "Invalid token."
        status_code = 401

    except ExpiredSignatureError:
        message = "Token expired."
        status_code = 403

    return {"message": message, "code": status_code}


@use_kwargs(PasswordResetSchema(), location="form")
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 403)
@marshal_with(DefaultResponseSchema, 401)
@marshal_with(DefaultResponseSchema, 404)
@http_code
def password_reset(token, **kwds):
    """
    Reset user password.
    """

    try:
        data = decode_token(token)
        email = data["identity"]
        if not data["user_claims"].get("token_password_reset"):
            raise DecodeError

        user = User.query.filter_by(email=email).first()
        passw = kwds.get("password")
        if user and passw:
            passw = generate_password_hash(passw)
            log = Log(updated_at=datetime.utcnow())
            user.logs.append(log)
            db_update(user, password=passw)
            message = "Successfully change password."
            status_code = 200
        else:
            message = "User not found."
            status_code = 404

    except DecodeError:
        message = "Invalid token."
        status_code = 401

    except ExpiredSignatureError:
        message = "Token expired."
        status_code = 403

    return {"message": message, "code": status_code}



from flask import Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    Function to load all middlewares.
    """

    dirname = "middlewares"
    middlewares = app.config.get(dirname.upper(), [])
    import_name = get_import_name(app)
    prefix = dirname + "."
    for name in middlewares:
        if not name.startswith(prefix):
            name = prefix + name

        name += ".init_middleware"
        try:
            middleware = import_attr(import_name + name)
        except (ImportError, AttributeError):
            middleware = import_attr(name.lstrip(prefix))

        app.wsgi_app = middleware(app.wsgi_app)



import os
from distutils.dir_util import copy_tree
from importlib import import_module
from types import ModuleType

from flask import Flask, current_app, render_template
from flask_sqlalchemy import Model

from .exception import ZemfrogModelNotFound, ZemfrogTemplateNotFound
from .globals import db

TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), "templates")


def get_template(*paths) -> str:
    """
    Function to get template base directory.

    :param paths: template directory or file name.

    :raises: ZemfrogTemplateNotFound

    """

    t = os.path.join(TEMPLATE_DIR, *paths)
    if not (os.path.isdir(t) or os.path.isfile(t)):
        raise ZemfrogTemplateNotFound("unknown template %r" % os.sep.join(paths))

    return t


def copy_template(name: str, dst: str):
    """
    Function for copying templates.

    :param name: template directory name.
    :param dst: Destination output.

    """

    t = get_template(name)
    copy_tree(t, dst)


def import_attr(module: str):
    """
    Functions to get attributes in modules.

    :param module: e.g. os.path

    """

    pkg, name = module.rsplit(".", 1)
    mod = import_module(pkg)
    return getattr(mod, name)


def search_model(name: str) -> str:
    """
    Function for getting the model location.

    :param name: model name.

    :raises: ZemfrogModelNotFound

    """

    for src, models in current_app.models.items():
        if name in models:
            return src

    raise ZemfrogModelNotFound("Cannot find %r orm model" % name)


def get_models(mod: ModuleType) -> list:
    """
    Function to get all ORM models in the module.

    :param mod: module object.

    """

    models = []
    childs = dir(mod)
    for c in childs:
        c = getattr(mod, c)
        try:
            if issubclass(c, Model):
                tbl_name = c.__name__
                models.append(tbl_name)
        except TypeError:
            pass

    return models


def db_add(model):
    """
    Functions for adding data to the database.
    """

    db.session.add(model)
    db_commit()


def db_delete(model):
    """
    Functions to delete data in the database.
    """

    db.session.delete(model)
    db_commit()


def db_update(model, **kwds):
    """
    Function to update data in database.
    """

    columns = get_column_names(model)
    for k, v in kwds.items():
        if k in columns:
            setattr(model, k, v)
    db_commit()


def db_commit():
    """
    Functions for saving data to a database.
    """

    db.session.commit()


def get_mail_template(name, **context):
    r"""
    Functions to get email templates.

    :param name: email template name.
    :param \*\*context: jinja context.

    """

    tmp = render_template("emails/" + name, **context)
    return tmp


def get_import_name(app: Flask) -> str:
    import_name = (
        ""
        if not app.import_name.endswith(".wsgi")
        else app.import_name.rstrip(".wsgi") + "."
    )
    return import_name


def get_user_roles(user):
    roles = {}
    for role in user.roles:
        permissions = [perm.name for perm in role.permissions]
        roles[role.name] = permissions
    return roles


def get_column_names(model):
    """
    Reference:
    * https://stackoverflow.com/questions/32927071/get-column-names-dynamically-with-sqlalchemy/33033067
    """

    columns = model.__table__.columns.keys()
    rels = model.__mapper__.relationships.keys()
    columns.extend(rels)
    return columns


def get_object_model(name):
    import_name = get_import_name(current_app)
    src = current_app.config[name.upper() + "_MODEL"]
    model = import_attr(import_name + src)
    return model



import click
from flask.cli import with_appcontext

from ..generator import g_loader


@click.group("loader")
def group():
    """
    Loader generator.
    """


@group.command()
@with_appcontext
@click.argument("name")
def new(name):
    """
    Create a new loader.
    """

    g_loader(name)


command = group



from flask import Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    A function to load all context processors into a flask.
    """

    contexts = app.config.get("CONTEXT_PROCESSORS", [])
    import_name = get_import_name(app)
    prefix = "contexts."
    for name in contexts:
        if not name.startswith(prefix):
            name = prefix + name

        name += ".init_context"

        try:
            func = import_attr(import_name + name)
        except (ImportError, AttributeError):
            func = import_attr(name.lstrip(prefix))

        app.context_processor(func)



from datetime import timedelta
from os import path

class Development(object):
    SECRET_KEY = "Your secret key!"
    SQLALCHEMY_DATABASE_URI = "sqlite:///" + path.join(
        path.dirname(__file__), "db.sqlite"
    )
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    JWT_SECRET_KEY = "JWT secret key!"
    JWT_ACCESS_TOKEN_EXPIRES = timedelta(days=7)
    MAIL_PORT = 8025
    MAIL_DEFAULT_SENDER = "admin@localhost.com"
    APISPEC_TITLE = "API Docs"
    APISPEC_SWAGGER_UI_URL = "/docs"
    APISPEC_SECURITY_DEFINITIONS = {
        "Bearer": {
            "type": "oauth2",
            "flow": "password",
            "tokenUrl": "/auth/jwt/login",
        }
    }
    APISPEC_SECURITY_PARAMS = [{"Bearer": []}]
    DEBUG = True
    {% if main_app -%}
        APPS = []
    {%- endif %}
    EXTENSIONS = [
        "zemfrog.extensions.sqlalchemy",
        "zemfrog.extensions.marshmallow",
        "zemfrog.extensions.migrate",
        "zemfrog.extensions.jwt",
        "zemfrog.extensions.mail",
        "zemfrog.extensions.celery",
        "zemfrog.extensions.apispec",
        "zemfrog.extensions.cors"
    ]
    COMMANDS = [
        "zemfrog.commands.api",
        "zemfrog.commands.blueprint",
        "zemfrog.commands.middleware",
        "zemfrog.commands.command",
        "zemfrog.commands.errorhandler",
        "zemfrog.commands.extension",
        "zemfrog.commands.model",
        "zemfrog.commands.task",
        "zemfrog.commands.user",
        "zemfrog.commands.role",
        "zemfrog.commands.permission",
        "zemfrog.commands.loader",
        "zemfrog.commands.secretkey",
        "zemfrog.commands.context",
        "zemfrog.commands.filter",
        {% if main_app -%}
            "zemfrog.commands.app"
        {%- endif %}
    ]
    BLUEPRINTS = ["auth"]
    STATICFILES = []
    MIDDLEWARES = []
    APIS = []
    ERROR_HANDLERS = {422: "api_errors", 400: "api_errors"}
    TASKS = []
    CONTEXT_PROCESSORS = []
    JINJA_FILTERS = []
    API_DOCS = True
    CREATE_DB = True
    USER_MODEL = "models.user.User"
    ROLE_MODEL = "models.user.Role"
    PERMISSION_MODEL = "models.user.Permission"
    LOADERS = [
        "zemfrog.loaders.extension",
        "zemfrog.loaders.staticfile",
        "zemfrog.loaders.model",
        "zemfrog.loaders.url",
        "zemfrog.loaders.blueprint",
        "zemfrog.loaders.middleware",
        "zemfrog.loaders.api",
        "zemfrog.loaders.error_handler",
        "zemfrog.loaders.command",
        "zemfrog.loaders.task",
        "zemfrog.loaders.openapi",
        "zemfrog.loaders.context_processor",
        "zemfrog.loaders.jinja_filter",
        "zemfrog.loaders.multiapp",
    ]
    CELERY_RESULT_BACKEND = "redis://127.0.0.1:6379"
    CELERY_BROKER_URL = CELERY_RESULT_BACKEND


class Production(Development):
    DEBUG = False
    JWT_COOKIE_SECURE = True


class Testing(Development):
    TESTING = True



import click
from flask.cli import with_appcontext

from ..helper import db_add, db_commit, db_delete, db_update, get_object_model


@click.group("permission")
def group():
    """
    Role permission manager.
    """


@group.command()
@with_appcontext
@click.argument("name")
@click.option("-d", "--description", required=True, help="Permission description.")
def new(name, description):
    """
    Create permission.
    """

    model = get_object_model("permission")
    perm = model.query.filter_by(name=name).first()
    if not perm:
        perm = model(name=name, description=description)
        db_add(perm)
        print("Successfully added %r permission" % name)
    else:
        print("%r permission already exists" % name)


@group.command()
@with_appcontext
@click.argument("name")
def remove(name):
    """
    Remove permission.
    """

    model = get_object_model("permission")
    perm = model.query.filter_by(name=name).first()
    if perm:
        db_delete(perm)
        print("Successfully deleted %r permission" % name)
    else:
        print("%r permission doesn't exist" % name)


@group.command()
@with_appcontext
@click.argument("name")
def update(name):
    """
    Update permission.
    """

    model = get_object_model("permission")
    perm = model.query.filter_by(name=name).first()
    if perm:
        cols = {}
        new_name = input("Enter a new permission name: ").strip()
        description = input("Enter a new permission description: ").strip()
        if not new_name:
            new_name = perm.name
        cols["name"] = new_name

        if not description:
            description = perm.description
        cols["description"] = description

        db_update(perm, **cols)
        print("Successfully updated the %r permission to %r" % (name, new_name))
    else:
        print("%r permission doesn't exist" % name)


@group.command()
@with_appcontext
def list():
    """
    Show permissions.
    """

    model = get_object_model("permission")
    perms = model.query.all()
    print("Total permissions: %d" % len(perms))
    for perm in perms:
        print("* %s - %s" % (perm.name, perm.description))


@group.command()
@with_appcontext
def drop():
    """
    Drop permissions.
    """

    print("Dropping all permissions... ", end="")
    model = get_object_model("permission")
    model.query.delete()
    db_commit()
    print("(done)")


command = group



from flask import Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    This function will load all urls in the main application.
    """

    import_name = get_import_name(app)
    routes = import_attr(import_name + "urls.routes")
    for url, view, methods in routes:
        app.add_url_rule(url, view_func=view, methods=methods)



from flask import Flask, send_from_directory


def loader(app: Flask):
    """
    This function is to create multiple static files based on the ``STATICFILES`` configuration.
    """

    staticfiles = app.config.get("STATICFILES", [])
    for static in staticfiles:
        path, endpoint, static_folder = static
        static_host = None
        if len(static) == 4:
            static_host = static[-1]

        def serve_static(filename):
            cache_timeout = app.get_send_file_max_age(filename)
            return send_from_directory(
                static_folder, filename, cache_timeout=cache_timeout
            )

        app.add_url_rule(
            path.rstrip("/") + "/<path:filename>",
            endpoint=endpoint,
            host=static_host,
            view_func=serve_static,
        )



"""
Reference: https://github.com/tiangolo/full-stack-flask-couchdb/tree/master/%7B%7Bcookiecutter.project_slug%7D%7D/backend/app/app/api/api_v1
"""

from apispec import APISpec
from apispec.ext.marshmallow import MarshmallowPlugin
from flask.app import Flask
from flask_apispec import FlaskApiSpec
from marshmallow import fields

FileField = fields.Raw


def init_app(app: Flask):
    global FileField

    docs = FlaskApiSpec(document_options=False)
    ma = MarshmallowPlugin()
    spec = APISpec(
        title=app.config.get("APISPEC_TITLE", "API Docs"),
        version=app.config.get("APISPEC_VERSION", "v1"),
        openapi_version=app.config.get("APISPEC_OAS_VERSION", "2.0"),
        plugins=[ma],
        securityDefinitions=app.config.get("APISPEC_SECURITY_DEFINITIONS", {}),
    )

    @ma.map_to_openapi_type("file", None)
    class FileField(fields.Raw):
        pass

    app.config.update({"APISPEC_SPEC": spec})
    docs.init_app(app)
    app.extensions["apispec"] = docs
